# MIT License
#
# Copyright (c) 2018 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from polygonetl.mappers.trace_mapper import EthTraceMapper


class EthSpecialTraceService(object):

    def __init__(self):
        self.trace_mapper = EthTraceMapper()

    def get_genesis_traces(self):
        from polygonetl.mainnet_genesis_alloc import MAINNET_GENESIS_ALLOC
        genesis_traces = [self.trace_mapper.genesis_alloc_to_trace(alloc)
                          for alloc in MAINNET_GENESIS_ALLOC]
        return genesis_traces

    def get_daofork_traces(self):
        from polygonetl.mainnet_daofork_state_changes import MAINNET_DAOFORK_STATE_CHANGES
        daofork_traces = [self.trace_mapper.daofork_state_change_to_trace(change)
                          for change in MAINNET_DAOFORK_STATE_CHANGES]
        return daofork_traces



import os

from setuptools import find_packages, setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


long_description = read("README.md") if os.path.isfile("README.md") else ""

setup(
    name="polygon-etl",
    version="0.3.3",
    author="Evgeny Medvedev",
    author_email="evge.medvedev@gmail.com",
    description="Tools for exporting Polygon blockchain data to CSV or JSON",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/blockchain-etl/polygon-etl",
    packages=find_packages(exclude=["schemas", "tests"]),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="polygon",
    # web3.py doesn't work on 3.5.2 and less (https://github.com/ethereum/web3.py/issues/1012)
    # google-cloud-pubsub==2.1.0 requires >=3.6 (https://pypi.org/project/google-cloud-pubsub/2.1.0/)
    # collections.Mapping unsupported in 3.10 (https://bugs.python.org/issue44737)
    python_requires=">=3.7,<3.10",
    install_requires=[
        "base58==2.1.1",
        "blockchain-etl-common==1.6.1",
        "click>=7.0",
        "eth-abi==1.3.0",
        "eth-utils==1.8.4",
        "ethereum-dasm==0.1.4",
        "requests>=2.23",
        "web3==4.7.2",
    ],
    extras_require={
        "streaming": [
            "google-cloud-pubsub==2.1.0",
            "pytz",  # See https://github.com/googleapis/python-pubsub/issues/468
            "google-cloud-storage==1.33.0",
            "pg8000==1.13.2",
            "pytz==2022.1",
            "sqlalchemy==1.3.13",
            "timeout-decorator==0.4.1",
        ],
    },
    entry_points={
        "console_scripts": [
            "polygonetl=polygonetl.cli:cli",
        ],
    },
    project_urls={
        "Bug Reports": "https://github.com/blockchain-etl/polygon-etl/issues",
        "Source": "https://github.com/blockchain-etl/polygon-etl",
    },
)



# MIT License
#
# Copyright (c) 2018 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class FailSafeExecutor:

    def __init__(self, delegate):
        self._delegate = delegate
        self._futures = []

    def submit(self, fn, *args, **kwargs):
        self._check_completed_futures()
        future = self._delegate.submit(fn, *args, **kwargs)
        self._futures.append(future)

        return future

    def shutdown(self):
        self._delegate.shutdown(wait=True)
        self._check_completed_futures()
        assert len(self._futures) == 0

    def _check_completed_futures(self):
        """Fail safe in this case means fail fast. TODO: Add retry logic"""
        for future in self._futures.copy():
            if future.done():
                # Will throw an exception here if the future failed
                future.result()
                self._futures.remove(future)
